;connect an oscillator
;select OSCILLATOR: HS if quartz used, watchdog:off, power up timer: on, code protect:off
;pin14 and pin4 to 5V
;pin5 to GND
;SCL on pin 8
;SDA on pin 9
;Choix RGN/YUV par pin 10 (RB4)

;ADV7300 driver for PIC16F84
;author: Vincent Brient, adaptation Laric, modifications Ki, support, Three-Eyes ("963sa HCFR" team)
;
;http://www.homecinema-fr.com/forum/viewtopic.php?t=29718349
;
;clock = 16MHz
;
; Configuration des sorties composantes de l'ADV7300 en RGB ou YUV au boot selon option
; Configuration du niveau de noirs en full range par modification dynamique des trames de configuration mises par
; le lecteur  l'ADV.
;
; Ce programme necessite un module equipe d'un PIC16F84 et d'un 4066 permettant d'ouvrir la ligne SDA de l'i2c 
; entre le lecteur et la carte progressive scan. La ligne SCL doit etre connectee en permanence independement
; du pilotage du 4066
;
; La configuration RGB/YUV se par le niveau appliqu sur la pin 10 :
; 	- pin 10 au niveau 1 pour la configuration RGB
; 	- pin 10 au niveau 0 pour la configuration YUV

;-------------- configuration du compilateur ----------------------------------------------
	LIST P=16f84a
	include "p16f84a.inc"	
	__config _CP_OFF & _PWRTE_ON & _WDT_OFF & _HS_OSC 
;-------------- definition des macros -----------------------------------------------------

pull_down_SCL	macro
	bcf	PI2C,SCL
	bsf	STATUS,RP0
	bcf	TI2C,SCL
	bcf	STATUS,RP0
	endm

float_SCL	macro
	bsf	STATUS,RP0
	bsf	TI2C,SCL
	bcf	STATUS,RP0
	endm

pull_down_SDA	macro
	bcf	PI2C,SDA
	bsf	STATUS,RP0
	bcf	TI2C,SDA
	bcf	STATUS,RP0
	endm

float_SDA	macro
	bsf	STATUS,RP0
	bsf	TI2C,SDA
	bcf	STATUS,RP0
	endm

startI2C	macro	; emission d'un bit de start sur l'i2c
	pull_down_SDA
	pull_down_SCL
	endm

stopI2C		macro	; emission d'un bit de stop
	pull_down_SDA
	float_SCL
	float_SDA
	endm
;--------------------------------------------------------------------------------------------------

;I2C constantes
PI2C		equ	PORTB
TI2C		equ	TRISB
SCL		equ	2	;SCL on pin 8
SDA		equ	3	;SDA on pin 9


I2C_address_write 	equ	0xD4
I2C_address_read 	equ	0xD5


;timer prescaler will be adjusted to 32
;timer duration = 4/(16MHz)*32*(256-timer_init) (.150 ->848S for 16MHz)
;duration must be longuer than 8/FreqI2Cmin (80S for 100KHz)
timer_init		equ	.150


;PIC constantes
RAM_START	EQU     0x0C	;first RAM address (0x0C for 16F84)

;variables in RAM
	ORG	RAM_START                        

count		res	1
temp1		res	1
temp2		res	1
temp3		res	1
temp4		res	1
registerI2C	res	1
count_msb	res	1
count_lsb	res	1
received_I2C_address	res	1
received_I2C_reg	res	1
received_I2C_data	res	1

	org	INDF			;boot address
	goto	begin

 	org	FSR			;retour d'interruption timer
	;mise a 1 du SDA et du SCL et fermeture du bus si ncessaire
	float_SDA
	float_SCL
	bsf 	PI2C,1
	goto	main_loop

;*************** subroutines ***********
;---------------- attente que le bus soit libre pendant 10 ms -----------------
wait_idle_I2C
	btfss	PI2C,SDA	; Attente que SDA soit  1
	goto	$-1

;Attente que SDA et SCL restent a 1 pendant 10 ms
check_10ms
	movlw	.22		;10ms / 446S = 22
	MOVWF	count_msb
check_short2
	movlw	.255
	MOVWF	count_lsb
loop_lsb2			; one loop_lsb cycle is 255*7 CPU cycles = 446S
	btfsc	PI2C,SDA	; retourne  Check_10ms si SDA = 0
	btfss	PI2C,SCL	; retourne  Check_10ms si SCL = 0
	goto	check_10ms

	decfsz	count_lsb,F ;
	goto	loop_lsb2

	decfsz	count_msb,F
	goto	check_short2
	return
;---------------- emission d'un octet sur l'i2c -----------------
sendI2C
	movwf	registerI2C
	movlw	.8
	movwf	count
loop_sendI2C
	rlf	registerI2C,F
	btfss	STATUS,C
	goto send0
	float_SDA		; Emission d'un 1
	goto sendclock
send0
	pull_down_SDA		; Emission d'un 0
sendclock
	float_SCL
	pull_down_SCL

	decfsz	count,F
	goto	loop_sendI2C

	float_SDA		; Mise a 1 du SDA pour l'Ack
	float_SCL
	pull_down_SCL
	return
; ----------------------------- emission de la trame de configuration en RGB -----------------------------
set_RGB_mode
	bcf 	PI2C,1		; ouverture du bus i2c pour ne pas perturber le lecteur pendant l'emission de trame

	startI2C
	movlw	I2C_address_write
	call	sendI2C	
	movlw	0x02
	call	sendI2C	
	movlw	0x98		;RGB mode Sync on all
	call	sendI2C	
	stopI2C	
	
	bsf 	PI2C,1		; fermeture du bus i2c
	return
;------------------------------- temporisations ----------------------------------------------------------

wait_1		movlw	.255		;wait 255*3*0,25S=191S
		movwf	temp1
		decfsz	temp1,F
		goto	$-1
		return	
wait_2					;wait 255*191S=48,77ms	
		movlw	.255	
		movwf	temp2
		call	wait_1
		decfsz	temp2,1
		goto	$-2
		return
wait_10s				;wait 205*48,77ms=10 s	
		MOVLW	.205
		MOVWF	temp3
		call	wait_2
		decfsz	temp3,1
		goto	$-2
		return
wait_4s					;wait 82**48,77ms=4 s	
		MOVLW	.82
		MOVWF	temp4
		call	wait_2
		decfsz	temp4,1
		goto	$-2
		return

;************** program ***************
begin

;IO init and settings

	clrf	PORTB
	bsf 	PI2C,1		; fermeture du bus i2c

	bsf	STATUS,RP0	; selection page memoire 1
	movlw	B'11111101'	; RB1 en sortie pour le pilotage du 4066
	movwf	TRISB
	movlw	b'10000100'	;timer prescaler = 32
	movwf	OPTION_REG
	movlw	b'00100000'	;enable timer interrupt
	movwf	INTCON
	bcf	STATUS,RP0	; selection page memoire 0

	float_SDA
	float_SCL

	call	wait_10s
	
	btfss	PORTB,4	; test du niveau d'entre de configuration YUV/RGB
	goto main_loop	; configuration YUV, pas d'mission de la trame set_RGB_mode
	call	wait_idle_I2C
	call	set_RGB_mode

main_loop
	movlw	timer_init
	movwf	TMR0		;initialisation du timer
	bcf	INTCON,T0IF	;reset du flag d'interruption sur TIMER
	bsf	INTCON,GIE	;activation des interruptions

	clrf	received_I2C_data
	clrf	received_I2C_address
	clrf	received_I2C_reg

; attente que le SDA et le SCL soient a 1
wait_SDA_SCL_set
	movwf	TMR0
	btfss	PI2C,SDA
	goto	wait_SDA_SCL_set
	btfss	PI2C,SCL
	goto	wait_SDA_SCL_set

; Attente d'un bit de start
wait_SDA_start
	movwf	TMR0
	btfsc	PI2C,SDA
	goto	wait_SDA_start	; attente d'un front descendant de SDA
	btfss	PI2C,SCL
	goto	wait_SDA_SCL_set	; ls SCL est a 0, ce n'etait pas un start

;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;**********************Boucle de lecture de l'adresse sur l'i2c********************
;*********** bit 7 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_address,7
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 6 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_address,6
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 5 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_address,5
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 4 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_address,4
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 3 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_address,3
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 2 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_address,2
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 1 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_address,1
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 0 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_address,0
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;********** Fin de lecture de l'adresse ************
;comparaison de l'adresse recue avec celle de l'ADV7300
	movf	received_I2C_address,W
	sublw	I2C_address_write
	btfss	STATUS,Z
	goto	main_loop		;retour a la boucle de scrutation si l'adresse est mauvaise

; attente ack
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;reinitialisation interruptions
	movlw	timer_init
	movwf	TMR0
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1

;**********************Boucle de lecture du numero de registre sur l'i2c********************
;*********** bit 7 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_reg,7
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 6 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_reg,6
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 5 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_reg,5
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 4 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_reg,4
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 3 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_reg,3
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 2 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_reg,2
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 1 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_reg,1
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 0 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_reg,0
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;********** Fin de lecture du numero de registre ************

;comparaison du numero de registre recu avec 0x10 (registre full range)
	movf	received_I2C_reg,W
	sublw	0x10
	btfss	STATUS,Z
	goto	main_loop		;retour a la boucle de scrutation si l'adresse est mauvaise

; attente ack
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;reinitialisation interruptions
	movlw	timer_init
	movwf	TMR0
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1

;*********** Boucle de lecture et de modification dynamique de la valeur ecrite dans le registre ********************
;*********** bit 7 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_data,7
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 6 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_data,6
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 5 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_data,5
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 4 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_data,4
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 3 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_data,3
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;*********** bit 2 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_data,2
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;******************************
; les deux derniers bits permettent de forcer le full range
; il faut forcer le bit 1  1 et le bit 0  0
	bcf 	PI2C,1		; ouverture du bus pour laisser le SDA  1 pour le bit 1
;*********** bit 1 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_data,1
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;******************************
	pull_down_SDA		; mise a 0 du SDA  1 pour le bit 0
;*********** bit 0 ************
;wait for SCL rising edge
	btfss	PI2C,SCL
	goto	$-1
;read address bit and store it
	btfsc	PI2C,SDA
	bsf	received_I2C_data,0
;wait for SCL falling edge
	btfsc	PI2C,SCL
	goto	$-1
;***** Fin de la boucle de lecture et modification dynamique de la valeur ecrite dans le registre ********
; remise a 1 du SDA et fermeture du bus pour l'ack
	float_SDA
	bsf 	PI2C,1

;*********************fin de la trame retour a la boucle de scrutation********************
	bcf	INTCON,GIE		;desactivation des interruptions
	goto	main_loop		;retour a la boucle de scrutation de l'i2c
	end